var json_url = php_vars.json_url;
var lic_url = php_vars.lic_url || "";
var show_buttons = php_vars.show_buttons || false;
var allow_print = php_vars.allow_print || false;
var allow_search = php_vars.allow_search || false;
var read_only = php_vars.read_only || false;
var plugin_url = php_vars.plugin_url || "";
var ajax_url = php_vars.ajax_url || "";
var mode2 = true;

var Diagram = MindFusion.Diagramming.Diagram;
var Behavior = MindFusion.Diagramming.Behavior;
var AbstractionLayer = MindFusion.AbstractionLayer;
var Alignment = MindFusion.Diagramming.Alignment;
var AnchorPattern = MindFusion.Diagramming.AnchorPattern;
var Shape = MindFusion.Diagramming.SimpleShape;
var Font = MindFusion.Drawing.Font;
var Rect = MindFusion.Drawing.Rect;
var Text = MindFusion.Drawing.Text;
var Thickness = MindFusion.Drawing.Thickness;
var Event = MindFusion.Diagramming.Events;
var GlassEffect = MindFusion.Diagramming.GlassEffect;
var Style = MindFusion.Diagramming.Style;
var SimpleShape = MindFusion.Diagramming.SimpleShape;
var ImageAlign = MindFusion.Diagramming.ImageAlign;
var LinkShape = MindFusion.Diagramming.LinkShape;

var id = 0;

MindFusion.Diagramming.CompatConfig.pixelThickness = true;

// creates a table node with the default settings
var OrgChartNode = function (parent, boss) {
	AbstractionLayer.initializeBase(OrgChartNode, this, [parent]);

	this.childNodes = [];

	// set up table cells
	this.redimTable(3, 4);
	this.getCell(1, 3).setColumnSpan(2);
	this.getCell(0, 0).setRowSpan(4);

	var font1 = new Font("Verdana", 3.5, true /*bold*/, false /*italic*/);
	var font2 = new Font("Verdana", 3, false /*bold*/, false /*italic*/);
	this.getCell(1, 0).setFont(font1);
	this.getCell(1, 1).setFont(font1);
	this.getCell(1, 2).setFont(font1);
	this.getCell(1, 3).setFont(font2);

	//allow text edit on cells with employee data
	this.getCell(0, 0).setTag("imageLocation");
	this.getCell(0, 0).editable = true;

	this.getCell(2, 0).setTag("fullName");
	this.getCell(2, 0).editable = true;

	this.getCell(2, 1).setTag("title");
	this.getCell(2, 1).editable = true;

	this.getCell(1, 3).setTag("comments");
	this.getCell(1, 3).editable = true;

	// customize appearance
	this.setCaptionHeight(0);
	this.setHandlesStyle(MindFusion.Diagramming.HandlesStyle.MoveOnly);
	this.setCellFrameStyle(MindFusion.Diagramming.CellFrameStyle.None);
	this.setBrush("purple");
	this.setShape(SimpleShape.RoundedRectangle);

	this.setFullName("Name");
	this.setTitle("title");
	this.setImageLocation(plugin_url + "/images/blank-profile.png");
	this.setId();
	this.setBoss(boss);
	this.setHierarchy();

	this.resize();
	this.setAnchorPattern(AnchorPattern.fromId("TopInBottomOut"));

	this.selectedStyle = new Style();
	this.selectedStyle.setStrokeThickness(6);
	this.selectedStyle.setStrokeDashStyle(MindFusion.Drawing.DashStyle.Dash);
	this.selectedStyle.setStroke("#2196F3");
}

OrgChartNode.prototype =
{
	// updates the existing elements
	updateCanvasElements: function (node) {
		this.setFields();
		AbstractionLayer.callBaseMethod(OrgChartNode, this, 'updateCanvasElements');
	},

	// gets the title of the employee
	getTitle: function () {
		return this.title;
	},

	// sets the title of the employee
	setTitle: function (value) {
		if (this.title !== value) {
			this.title = value;
			this.invalidate();
		}
	},

	// gets the full name of the employee
	getFullName: function () {
		return this.fullName;
	},

	// sets the full name of the employee
	setFullName: function (value) {
		if (this.fullName !== value) {
			this.fullName = value;
			this.invalidate();
		}
	},

	// gets the url where the employee picture is located
	getImageLocation: function () {
		return this.nodeImageLocation;
	},

	// sets the url where the employee picture is located
	setImageLocation: function (value) {
		if (this.nodeImageLocation != value) {   //places the image in cell (0,0)
			var alignment = MindFusion.Diagramming.ImageAlign;
			this.nodeImageLocation = value;
			this.getCell(0, 0).setImageLocation(value);
			this.getCell(0, 0).setImageAlign(alignment.Stretch);
			this.invalidate();
		}
	},

	// gets the comments about the employee
	getComment: function () {
		return this.comments;
	},

	// gets the comments about the employee
	setComment: function (value) {
		if (this.comments !== value) {
			this.comments = value;
			this.invalidate();
		}
	},

	// assigns an id to the employee
	setId: function () {
		this.id = id;
		id++;
	},

	// gets the id that was assigned to the employee
	getId: function () {
		return this.id;
	},

	// assigns the employee data to the table cells
	setFields: function () {
		// hide the caption and place the employee names on row 0 
		this.setCaptionHeight(0);
		this.getCell(1, 0).setText("Name:")
		this.getCell(2, 0).setText(this.fullName);

		// the employee title is on the second row
		this.getCell(1, 1).setText("Title:");
		this.getCell(2, 1).setText(this.title);

		// render the employee id at the third row
		this.getCell(1, 2).setText("ID: " + this.id);
		// place the image in cell (0, 0)
		this.getCell(0, 0).setImagePadding(new Thickness(2, 2, 2, 2));
		this.getCell(0, 0).setImageLocation(this.nodeImageLocation);
		this.getCell(0, 0).setImageAlign(ImageAlign.Fit);

		// sets the comments for the employee
		this.getCell(1, 3).setText(this.comments);

		// rearrange the org hierarchy
		this.setHierarchy();
		this.setColor();
	},

	// resize the node to guarantee that all data would fit
	resize: function () {
		var oldHeight = this.bounds.height;
		var oldImageWidth = this.getColumn(0).width;

		//resize the node to fit the text
		this.resizeToFitText(true, true);

		//add or remove the necessary space to fit the image well
		var newHeight = this.bounds.height - oldHeight;
		this.getCell(0, 0).imagePadding.top += newHeight / 2;
		this.getCell(0, 0).imagePadding.bottom += newHeight / 2;
		this.getColumn(0).width = oldImageWidth;
		this.bounds.width += oldImageWidth;
	},

	// assigns the specified boss as the employee's boss
	setBoss: function (boss) {
		if (boss != this.boss) {
			this.boss = boss;
			this.setHierarchy();
			this.invalidate();
		}
	},

	// rebuilds the hierarchy
	setHierarchy: function () {
		// the ceo has no boss
		if (this.boss == undefined) {
			this.hierarchy = 0;
		}
		else {
			// first level of executives under the boss
			if (this.boss.hierarchy == undefined) {
				this.hierarchy = 1;
			}
			else {
				// increase the depth of the hierarchy
				this.hierarchy = this.boss.hierarchy + 1;
				this.boss.addChild(this);
			}
		}
		// rearrange the hierarchy
		for (var i = 0; i < this.childNodes; i++)
			this.childNodes[i].setHierarchy();

		this.setColor();
	},

	// picks up a color based on the hierarchy level
	setColor: function () {
		var hierarchyLevel = this.hierarchy % 6;
		switch (hierarchyLevel) {
			case 0:
				this.setBrush("#ea684f");
				break;
			case 1:
				this.setBrush("#e0e9e9");
				break;
			case 2:
				this.setBrush("#9caac6");
				break;
			case 3:
				this.setBrush("#669acc");
				break;
			case 4:
				this.setBrush("#678b99");
				break;
			case 5:
				this.setBrush("#616a7f");
				break;
			case 6:
				this.setBrush("#c0c0c0");
				break;
			default:
				this.setBrush("#c0c0c0");
				break;
		}
	},

	// adds a new employee
	addChild: function (child) {
		//checks if the employee already exists
		var isHere = false;
		for (var i = 0; i < this.childNodes.length; i++) {
			if (this.childNodes[i].id == child.id)
				isHere = true;
		}
		if (!isHere)
			this.childNodes.push(child);
	},

	// removes the specified child from the collection of nodes
	removeChild: function (child) {
		var index = this.childNodes.indexOf(child);
		if (index > -1)
			this.childNodes.splice(index, 1);
	},

	// rearranges the hierarchy
	resetHierarchy: function () {
		this.setHierarchy();
		for (var i = 0; i < this.childNodes.length; i++) {
			this.childNodes[i].setBoss(this);
			this.childNodes[i].resetHierarchy();
		}
	},

	// checks if the specified node is a child node
	isChild: function (child) {
		for (var i = 0; i < this.childNodes.length; i++) {
			if (child === this.childNodes[i])
				return true;
		}
		return false;
	},

	getData: function () {
		var data = this.fullName + " " + this.title + " " + this.comments;
		return data.toLowerCase();
	}
};

var tree = null;

document.addEventListener('DOMContentLoaded', function () {

	AbstractionLayer.registerClass(OrgChartNode, "OrgChartNode", MindFusion.Diagramming.TableNode);

	// we apply the tree layout to arrange the diagram
	tree = new MindFusion.Graphs.TreeLayout();

	// customize the tree layout
	tree.direction = MindFusion.Graphs.LayoutDirection.TopToBottom;
	tree.linkType = MindFusion.Graphs.TreeLayoutLinkType.Cascading;

	var canvases = document.querySelectorAll("canvas");
	for (var i = 0; i < canvases.length; i++) {
		if (canvases[i].id.indexOf("mf_diagram") > -1)
			initDiagram(canvases[i]);
	}
});

function initDiagram(element) {

	// create the diagram object and apply customization
	element.oncontextmenu = function () { return false; }
	var diagram = Diagram.create(element);
	diagram.setLicenseLocation(lic_url);
	diagram.setBehavior(Behavior.Modify);
	diagram.setDelKeyAction(MindFusion.Diagramming.DelKeyAction.DeleteSelectedItems);
	diagram.setShowGrid(false);
	diagram.setAllowSelfLoops(false);
	diagram.setAutoScroll(true);
	diagram.setLinkShape(LinkShape.Cascading);
	diagram.setBackBrush('#ffffff');
	diagram.setVirtualScroll(false);
	checkMode(diagram);

	//add an EventListener for clicked events
	diagram.addEventListener(Event.clicked, function (diagram, eventArgs) {
		clearSelected(diagram);

		if (read_only) return;

		// check which mouse button was clicked
		var button = eventArgs.getMouseButton();
		var position = eventArgs.getMousePosition();

		// click with the right mouse button creates a node
		if (button === 2) {
			var node = new OrgChartNode(diagram, undefined);
			node.setBounds(new Rect(position.x, position.y, 20, 20));
			node.resize();

			// adds the node to the diagram items
			diagram.addItem(node);

			// rearrange the diagram
			diagram.arrangeAnimated(tree);
		}
	});

	diagram.addEventListener(Event.animatedLayoutCompleted, function (diagram, eventArgs) {
		// resize the diagram to fit all items
		diagram.resizeToFitItems();
	});

	diagram.addEventListener(Event.nodeClicked, onNodeClicked);

	// add an EventListener for nodeSelected events
	diagram.addEventListener(Event.nodeSelected, function (diagram, eventArgs) {
		if (read_only) return;
		// make sure the clicked node appears above any other nodes at the same location
		eventArgs.getNode().setZIndex(700);
	});

	// raised when the user deselcts a node
	diagram.addEventListener(Event.nodeDeselected, function (diagram, eventArgs) {
		//reset the z-index of the node
		eventArgs.getNode().setZIndex(0);
	});

	//add an eventListener for the linkCreated event
	diagram.addEventListener(Event.linkCreated, onLinkCreated);

	//add an eventListener for the cellTextEdited event
	diagram.addEventListener(Event.cellTextEdited, function (diagram, cellArgs) {
		cellArgs.getCell().onEdited(diagram, cellArgs);
	});

	// add an EventLitener for the linkCreated event
	diagram.addEventListener(Event.linkCreated, function (diagram, eventArgs) {
		diagram.arrangeAnimated(tree);
	});

	// add an eventListener for the nodeDeleted event
	diagram.addEventListener(Event.nodeDeleted, function (diagram, eventArgs) {
		diagram.arrangeAnimated(tree);
	})

	// add an eventListener for the linkDeleted event 
	diagram.addEventListener(Event.linkDeleted, function (diagram, eventArgs) {
		// gets the destinationNode for the link that is deleted
		var destNode = eventArgs.getLink().destination;

		// remove the destinationNode as a subordinate to the boss of the destination node  
		destNode.boss.removeChild(destNode);

		// destinationNode now has no boss
		destNode.boss = undefined;

		// reset the hierarchy for the destination node
		destNode.resetHierarchy();
		destNode.getCell(2, 2).setText("");
		diagram.arrangeAnimated(tree);
	});

	// handles nodeModified events
	diagram.addEventListener(Event.nodeModified, function (diagram, eventArgs) {
		// if the modified node does not have a boss
		if (!(eventArgs.getNode().boss)) {
			// get the mouse position
			var mousePoint = eventArgs.getMousePosition();
			// and all nodes at this position
			var nodes = diagram.getNodesAt(mousePoint);

			for (var i = 0; i < nodes.length; i++) {
				// if the node is not the one being modified and is not child of the one being modified
				if (nodes[i] !== eventArgs.getNode() && !eventArgs.getNode().isChild(nodes[i])) {
					// set nodes[i] as boss for the modified node
					eventArgs.getNode().setBoss(nodes[i]);

					// create a diagramLink to show the hierarchy
					var l = diagram.getFactory().createDiagramLink(nodes[i], eventArgs.getNode());

					// apply link design
					styleLink(l);

					// reset the node hierarchy
					eventArgs.getNode().resetHierarchy();

					// rearrange the diagram
					diagram.arrangeAnimated(tree);

					// resize the diagram to fit all items
					diagram.resizeToFitItems();
					break;
				}
			}
		}
	})

	//add an EventListener for inPlaceEdit events 
	diagram.addEventListener(Event.enterInplaceEditMode, function (diagram, eventArgs) {
		if (read_only) return;

		var cell = eventArgs.getItem().getCellKind();
		if (cell.image)
			eventArgs.getControl().setAttribute("placeholder", "Image URL:");
		else
			eventArgs.getControl().setAttribute("placeholder", cell.getTag() + ":");
	});

	loadData(diagram);

	// disable built-in inplace editing (here it's triggered by different events than double-click)
	diagram.setAllowInplaceEdit(false);

	if (show_buttons) {
		element.parentNode.parentNode.querySelector(".mf_btnLoad").addEventListener("click", onLoadClick);
		element.parentNode.parentNode.querySelector(".mf_btnSave").addEventListener("click", onSaveClick);
	}
	if (allow_print) {
		element.parentNode.parentNode.querySelector(".mf_btnPrint").addEventListener("click", onPrintClick);
	}
	if (allow_search) {
		element.parentNode.parentNode.querySelector(".mf_btnSearch").addEventListener("click", onSearchClick);
		element.parentNode.parentNode.querySelector(".mf_searchresult").addEventListener("click", function (e) {
			e.target.classList.remove("mf_open");
			e.target.innerHTML = "";
		});
	}
}

// called when the user edits a table
function editNode(diagram, eventArgs) {
	var cellEditor = eventArgs.getNode().cellFromPoint(eventArgs.getMousePosition());

	// the table node to edit
	var tableNode = eventArgs.getNode();

	var edit = false;

	// cells that cannot be edited have no cellEditor assigned
	if (cellEditor.cell != undefined)
		if (cellEditor.cell.editable == true)
			edit = true;

	// if the cell can be edited
	if (edit) {
		diagram.beginEdit(eventArgs.getNode(), eventArgs.getMousePosition());
		//$("diagram_inplaceInput").attr("placeholder", "url");

		cellEditor.cell.onEdited = function (diagram, tableCell) {
			if (edit) {
				if (cellEditor.cell.image) {
					// read and assign the URL of the new image
					if (tableCell.getNewText() != undefined &&
						tableCell.getNewText() != "" &&
						tableCell.getNewText != "undefined") {
						tableNode.setImageLocation(tableCell.getNewText());
						cellEditor.cell.text.height = 0;
						cellEditor.cell.text.text = "";
					}
				}
				//resize the table cell to fit the new image
				else if (cellEditor.row < 3) {
					var oldImageWidth = tableNode.getColumn(0).width;
					tableNode.resizeToFitText(true, true);
					tableNode.getColumn(0).width = oldImageWidth;
					tableNode.bounds.width += oldImageWidth;
				}

				else if (cellEditor.row === 3) {
					var oldHeight = tableNode.bounds.height;
					var oldImageWidth = tableNode.getColumn(0).width;
					tableNode.resizeToFitText(true, true);
					var newHeight = tableNode.bounds.height - oldHeight;
					tableNode.getCell(0, 0).imagePadding.top += newHeight / 2;
					tableNode.getCell(0, 0).imagePadding.bottom += newHeight / 2;
					tableNode.getColumn(0).width = oldImageWidth;
					tableNode.bounds.width += oldImageWidth;
				}

				var tag = cellEditor.cell.getTag();
				if (tag) {
					tableNode[tag] = tableCell.newText;
				}

				diagram.arrangeAnimated(tree);
			}
		}
	}

	// checks to see if the edited object is an image
	tableNode.getCellKind = function () {
		return cellEditor.cell;
		//if (cellEditor.cell.image)
		//	return 0;
		// if (cellEditor.row == 0)
		// 	return 1;
		// if (cellEditor.row == 1)
		// 	return 2;
		// if (cellEditor.row == 3)
		// 	return 3;
		// return 4;
	}
}

// raised when the user creates a link
function onLinkCreated(diagram, eventArgs) {
	if (eventArgs.getLink().permission === true)
		styleLink(eventArgs.getLink());
}

function styleLink(link) {
	link.setPen("#2d3956");
	link.setStrokeThickness(2.0);
	link.setHeadShape();
}

// raised when the user creates a node
function createNode(diagram, eventArgs) {
	var parent = eventArgs.getNode();
	var node = new OrgChartNode(diagram, parent);
	node.setBounds(parent.getBounds());

	diagram.addItem(node);
	node.setComment("(comment)");
	node.resize();

	// create a link that connects the new node
	var link = diagram.getFactory().createDiagramLink(eventArgs.getNode(), node);
	link.permission = true;

	// apply the link design
	styleLink(link);

	// rearrange the diagram
	diagram.arrangeAnimated(tree);

	// resize the diagram to fit all nodes
	diagram.resizeToFitItems();
}

// raised when the user clicks on a node
function onNodeClicked(diagram, eventArgs) {
	clearSelected(diagram);
	if (read_only) return;

	// checks if the user has clicked with the left mouse button (0)
	var button = eventArgs.getMouseButton();
	if (button === 0)
		editNode(diagram, eventArgs);

	//click with the right mouse button creates a node
	else if (button === 2)
		createNode(diagram, eventArgs);
}

function setLinks(diagram) {
	for (var i = 0; i < diagram.nodes.length; i++) {
		if (diagram.nodes[i].boss !== undefined) {
			var link = diagram.getFactory().createDiagramLink(diagram.nodes[i].boss, diagram.nodes[i]);
			styleLink(link);
		}
	}

	// arrange the diagram with the TreeLayout
	diagram.arrangeAnimated(tree);
}

// serialization
function saveJson(diagram) {
	var result = [];
	for (var i = 0, l = diagram.nodes.length; i < l; i++) {
		if (diagram.nodes[i].boss === undefined) {
			result.push(saveNode(diagram.nodes[i]));
		}
	}

	var json = {
		items: result
	}

	return AbstractionLayer.toJson(json);
}

function saveNode(node) {
	var json = {
		id: node.id,
		title: node.title,
		fullName: node.fullName,
		imageLocation: node.nodeImageLocation
	}
	if (node.comments != "(comment)") json.comments = node.comments;

	var children = [];
	for (var i = 0, l = node.childNodes.length; i < l; i++) {
		children.push(saveNode(node.childNodes[i]));
	}

	json.descendants = children;

	return json;
}

function loadJson(diagram, json) {
	diagram.clearAll();

	var obj = AbstractionLayer.fromJson(json);

	if (obj.items instanceof Object) {
		for (var i = 0; i < obj.items.length; i++) {
			if (obj.items[i]) {
				loadNode(diagram, obj.items[i]);
			}
		}
	}
}

function loadNode(diagram, json) {
	var node = new OrgChartNode(diagram);

	node.id = json.id;
	node.title = json.title;
	node.fullName = json.fullName;
	node.nodeImageLocation = json.imageLocation;
	node.comments = json.comments;

	diagram.addItem(node);

	node.updateCanvasElements();
	node.resize();

	if (json.descendants instanceof Object) {
		for (var i = 0; i < json.descendants.length; i++) {
			if (json.descendants[i]) {
				var child = loadNode(diagram, json.descendants[i]);
				child.setBoss(node);
			}
		}
	}

	return node;
}

function checkMode(diagram) {
	if (lic_url.replace(/^.*[\\\/]/, '') == "diagram_lic.txt") {
		getJSON(lic_url, function (err, data) {
			if (err != null) {
			} else {
				if (data) mode2 = false;
				if (mode2) read_only = true;

				// set diagram behavior
				if (read_only) {
					diagram.setBehavior(Behavior.DoNothing);
					diagram.setDelKeyAction(MindFusion.Diagramming.DelKeyAction.None);
				}
				else {
					diagram.setBehavior(Behavior.Modify);
					diagram.setDelKeyAction(MindFusion.Diagramming.DelKeyAction.DeleteSelectedItems);
				}
			}
		});
	}
}

function getJSON(url, callback) {
	var xhr = new XMLHttpRequest();
	xhr.open('GET', url, true);

	xhr.onload = function () {
		var status = xhr.status;
		if (status == 200) {
			callback(null, xhr.response);
		} else {
			callback(status);
		}
	}
	xhr.send();
}

function loadData(diagram) {
	var url = diagram.get_element().getAttribute("data-json-url") || json_url;
	if (!url) return;

	getJSON(url, function (err, data) {
		if (err != null) {
			console.error(err);
		} else {
			loadJson(diagram, data);
			setLinks(diagram);
		}
	});
}

function onLoadClick(e) {
	var diagram = Diagram.find(e.target.parentNode.parentNode.querySelector("canvas").id, e.target.parentNode.parentNode);

	loadData(diagram);
}

function onSaveClick(e) {
	if (mode2) return;

	var diagram = Diagram.find(e.target.parentNode.parentNode.querySelector("canvas").id, e.target.parentNode.parentNode);
	var json = saveJson(diagram);

	var allow_upload = (diagram.get_element().getAttribute("data-allow-upload") == "true");
	if (allow_upload) {
		var url = diagram.get_element().getAttribute("data-json-url");
		var json_file_name = url.substring(url.lastIndexOf('/') + 1);

		postJSON(json, json_file_name, function (data) {
			console.log(data.message);
		});
	}
	else
		download(json, 'diagram.json', 'text/plain');
}

function postJSON(json, json_file_name, callback) {

	var str = {
		'action': 'mfdiagram_upload_json',
		'json_file_name': json_file_name,
		'data': json
	};
	jQuery.ajax({
		type: "POST",
		url: ajax_url,
		data: str,
		success: function (data) {
			callback(data);
		},
		error: function (data) {
			callback(data);
		}
	});
}

function download(text, name, type) {
	if (text != null && navigator.msSaveBlob)
		return navigator.msSaveBlob(new Blob([text], { type: type }), name);

	var a = document.createElement("a");
	var file = new Blob([text], { type: type });
	var url = URL.createObjectURL(file);
	a.href = url;
	a.download = name;
	document.body.appendChild(a);
	a.click();
	window.URL.revokeObjectURL(url);
	document.body.removeChild(a);
}

function onPrintClick(e) {
	if (mode2) return;

	var diagram = Diagram.find(e.target.parentNode.parentNode.querySelector("canvas").id, e.target.parentNode.parentNode);

	var printWindow = window.open();
	var imgSrc = diagram.get_element().toDataURL("image/png", "");
	var img = printWindow.document.createElement("img");
	img.src = imgSrc;
	img.addEventListener("load", function () {
		printWindow.document.body.appendChild(img);
		printWindow.print();
		printWindow.close();
	});
}

function onSearchClick(e) {
	var diagram = Diagram.find(e.target.parentNode.parentNode.querySelector("canvas").id, e.target.parentNode.parentNode);

	clearSelected(diagram);

	var resultDiv = diagram.get_element().parentNode.parentNode.querySelector(".mf_searchresult");

	var searchStr = e.target.parentElement.querySelector(".mf_searchbox").value.trim().toLowerCase();

	if (searchStr) {
		resultDiv.classList.add("mf_open");

		var nodes = searchLocal(diagram, searchStr);
		if (nodes.length > 0) {
			for (var i = 0; i < nodes.length; i++) {
				diagram.selection.removeItem(nodes[i]);
				nodes[i].setStyle(nodes[i].selectedStyle);
			}
			diagram.scrollTo(nodes[0].getBounds().x, nodes[0].getBounds().y);

			resultDiv.innerHTML = nodes.length + " node(s) found.<br/>";
		}
		if (nodes.length == 0)
			resultDiv.innerHTML = "No results found."
	}
}

function searchLocal(diagram, str) {
	var result = [];
	for (var i = 0; i < diagram.nodes.length; i++) {
		if (diagram.nodes[i].getData().indexOf(str) > -1) {
			result.push(diagram.nodes[i]);
		}
	}
	return result;
}

function clearSelected(diagram) {
	for (var i = 0; i < diagram.nodes.length; i++) {
		diagram.nodes[i].setStyle(null);
	}
	var resultDiv = diagram.get_element().parentNode.parentNode.querySelector(".mf_searchresult");
	if (resultDiv) {
		resultDiv.classList.remove("mf_open");
		resultDiv.innerHTML = "";
	}
}