<?php
/*
Plugin Name: Org Chart
Plugin URI: http://mindfusion.eu
Description: The easy way to create and modify an org chart in WordPress. Supports images, loads from a json file.
Author: MindFusion LLC.
Version: 1.1.0
Author URI: http://mindfusion.eu
*/

function mfdiagram_add_scripts($dep) {
	wp_enqueue_style( 'mfdiagram-plugin-style',  plugins_url( 'style/mfdiagram-style.css', __FILE__ ));

    wp_enqueue_script( 'mfcommon-script', plugins_url( 'js/MindFusion.Common.js', __FILE__ ),  array(), '1.0.0', true );
    wp_enqueue_script( 'mfdiagramming-script', plugins_url( 'js/MindFusion.Diagramming.js', __FILE__ ),  array(), '1.0.0', true );
    wp_enqueue_script( 'mfdiagram-plugin-script', plugins_url( 'js/mfdiagram-plugin.js', __FILE__ ),  array(), '1.0.0', true );

	$options = get_option( 'mfdiagram_settings' );
	
	$json_url = $options['path_json'];

    $data = array(
		'json_url' => $json_url,
		'lic_url' => $options['path_lic'],
		'show_buttons' => $options['show_buttons'],
		'read_only' => $options['read_only'],
		'allow_print' => $options['allow_print'],
		'allow_search' => $options['allow_search'],
		'plugin_url' => plugins_url('', __FILE__ ),
		'ajax_url' => admin_url('admin-ajax.php')
	);
	
    wp_localize_script( 'mfdiagram-plugin-script', 'php_vars', $data);
}

function mfdiagram_activate() {
  
    $upload = wp_upload_dir();
    $upload_dir = trailingslashit($upload['basedir']) . 'orgcharts';
	wp_mkdir_p( $upload_dir );
}

register_activation_hook( __FILE__, 'mfdiagram_activate' );

add_action('wp_enqueue_scripts', 'mfdiagram_add_scripts' );
add_action('wp_ajax_mfdiagram_upload_json', 'mfdiagram_upload_json');
add_action('wp_ajax_nopriv_mfdiagram_upload_json', 'mfdiagram_upload_json');

function mfdiagram_upload_json($json){   
	if ( is_user_logged_in() ) {
		$json = $_POST['data'];

		$upload = wp_upload_dir();
		$upload_dir = trailingslashit($upload['basedir']) . 'orgcharts';

		$json_file_name = $_POST['json_file_name'];

		$filename = trailingslashit($upload_dir) . $json_file_name;
		$file = @fopen($filename, "w") or die("Unable to open file!" . $filename);
		$write = @fputs($file, stripslashes_deep($json)); 
		@fclose($file);
 
		wp_send_json( array( 
			'success' => true, 
			'message' => 'Changes to ' . $filename . ' uploaded.' ) 
		);
	}
	else 
	{
		wp_send_json( array( 
			'success' => false, 
			'message' => 'Uploads are not allowed for visitors. Please log in to upload files.' ) );
		//$error = new WP_Error( '401', 'Unauthorized', 'Uploads are not allowed for visitors. Please log in to upload files.' );
		//wp_send_json_error( $error );
	}
}

function mfdiagram_render($atts) {

	$options = get_option( 'mfdiagram_settings' );

	$defaults = array( 
		'json_url' => '',
		'allow_upload' => false
	 );

	$atts = shortcode_atts( $defaults, $atts );
	$local_json = $atts['json_url'];

	$allow_upload = $atts['allow_upload'];
	if( $allow_upload)
	{
		$upload = wp_upload_dir();
		$upload_dir = $upload['baseurl'] . '/orgcharts/';
		$post_slug = get_post_field( 'post_name', get_post() ); 
		$post_id = get_the_ID();
		$local_json = $upload_dir . $post_slug . $post_id . ".json"; 
	}

	$diagram_id = uniqid("mf_diagram_");

	echo '<div class="mf_wrapper">';

	if( $options['allow_search'])
	{
		echo '<div class="mf_toolbar">';
		echo '<input class="mf_searchbox" type="text" placeholder="Search ...">';
		echo '<button type="button" class="mf_btnSearch" value="Search">Go</button>';
		echo '</div>';
		echo '<div class="mf_searchresult"></div>';
	}

	if( $options['show_buttons'] || $options['allow_print'] )
	{
		echo '<div class="mf_toolbar">';
		if( $options['show_buttons'])
		{
			echo '<button type="button" class="mf_btnSave" value="Save">Save</button>';	
			echo '<button type="button" class="mf_btnLoad" value="Load">Load</button>';
		}
		if( $options['allow_print'])
		{
			echo '<button type="button" class="mf_btnPrint" value="Print">Print</button>';	
		}
		echo '</div>';
	}

    echo '<div style="overflow: auto;">';
    echo '<canvas id="'.$diagram_id.'" data-json-url="'.$local_json.'" data-allow-upload="'.$allow_upload.'" width="1252" height="551" style="cursor: default;">';
    echo 'This plugin requires a browser that supports HTML 5 Canvas element.';
    echo '</canvas>';
	echo '</div>';
	
	echo '</div>';
}

function mfdiagram_shortcode($atts) {
    ob_start();

    mfdiagram_render($atts);

    return ob_get_clean();
}

add_shortcode( 'mindfusion_org_chart', 'mfdiagram_shortcode' );


/**
 * Register the plugin options
 */
function mfdiagram_register_settings() {
	register_setting( 'mfdiagram_settings_group', 'mfdiagram_settings' );
}
add_action( 'admin_init', 'mfdiagram_register_settings' );

/**
 * Register the settings page
 */
function mfdiagram_settings_menu() {
	add_options_page( __( 'MindFusion Org Chart' ), __( 'MindFusion Org Chart Settings' ), 'manage_options', 'mfdiagram_settings', 'mfdiagram_settings_page' );
}
add_action( 'admin_menu', 'mfdiagram_settings_menu' );
/**
 * Render the settings page
 */
function mfdiagram_settings_page() {
	$options = get_option( 'mfdiagram_settings' ); ?>
	<div class="wrap">
		<h2><?php screen_icon(); _e('MindFusion Org Chart Plugin Settings'); ?></h2>
		<form method="post" action="options.php" class="options_form">
			<?php settings_fields( 'mfdiagram_settings_group' ); ?>
			<table class="form-table">
				<tr valign="top">
					<th scop="row">
						<label for="mfdiagram_settings[path_json]"><?php _e( 'Path to org chart JSON data' ); ?></label>
					</th>
					<td>
						<input class="regular-text" type="text" id="mfdiagram_settings[path_json]" style="width: 300px;" name="mfdiagram_settings[path_json]" value="<?php if( isset( $options['path_json'] ) ) { echo esc_attr( $options['path_json'] ); } ?>"/>
						<p class="description"><?php _e( 'Enter the URL to load org chart JSON data from.'); ?></p>
					</td>
				</tr>
				<tr>
					<th scop="row">
						<label for="mfdiagram_settings[path_lic]"><?php _e( 'Path to license file' ); ?></label>
					</th>
					<td>
						<input class="regular-text" type="text" id="mfdiagram_settings[path_lic]" style="width: 300px;" name="mfdiagram_settings[path_lic]" value="<?php if( isset( $options['path_lic'] ) ) { echo esc_attr( $options['path_lic'] ); } ?>"/>
						<p class="description"><?php _e( 'Enter the path to the licensing file.' ); ?></p>
					</td>
				</tr>
				<tr>
					<th scop="row">
						<label for="mfdiagram_settings[show_buttons]"><?php _e( 'Show save and load buttons' ); ?></label>
					</th>
					<td>
						<input name="mfdiagram_settings[show_buttons]" type="checkbox" value="1" <?php checked( $options['show_buttons'] ); ?> />
					</td>
				</tr>
				<tr>
					<th scop="row">
						<label for="mfdiagram_settings[allow_print]"><?php _e( 'Show print button' ); ?></label>
					</th>
					<td>
						<input name="mfdiagram_settings[allow_print]" type="checkbox" value="1" <?php checked( $options['allow_print'], 1 ); ?> />
					</td>
				</tr>
				<tr>
					<th scop="row">
						<label for="mfdiagram_settings[allow_search]"><?php _e( 'Show search button' ); ?></label>
					</th>
					<td>
						<input name="mfdiagram_settings[allow_search]" type="checkbox" value="1" <?php checked( $options['allow_search'], 1 ); ?> />
					</td>
				</tr>
				<tr>
					<th scop="row">
						<label for="mfdiagram_settings[read_only]"><?php _e( 'Read-only mode' ); ?></label>
					</th>
					<td>
						<input name="mfdiagram_settings[read_only]" type="checkbox" value="1" <?php checked( $options['read_only'], 1 ); ?> />
					</td>
				</tr>
			</table>
			<?php submit_button(); ?>
		</form>
	</div><!--end .wrap-->
	<?php
}
/**
 * Add a widget to the dashboard.
 */
function mfdiagram_add_dashboard_widget() {
	wp_add_dashboard_widget(
		'mfdiagram_dashboard_widget',         // Widget slug.
		'MindFusion Org Chart',         // Title.
		'mfdiagram_widget' // Display function.
	);	
}
add_action( 'wp_dashboard_setup', 'mfdiagram_add_dashboard_widget' );
	
/**
* Create the function to output the contents of the widget.
*/
function mfdiagram_widget() {
		mfdiagram_settings_page();
}

/**
* Add a settings link.
*/
function mfdiagram_add_settings_link( $links ) {
	$settings_link = '<a href="options-general.php?page=mfdiagram_settings">' . __( 'Settings' ) . '</a>';
	array_unshift($links, $settings_link);
	return $links;
}
$plugin = plugin_basename( __FILE__ );
add_filter( "plugin_action_links_$plugin", 'mfdiagram_add_settings_link' );



